@@ -5,7 +5,7 @@ Command Parser
 """
 
 from .config import CommandConfig
-from .commands import Command
+from .commands import Command, main
 from .context import Context, get_current_context, ctx
 from .exceptions import *  # noqa
 from .error_handling import ErrorHandler, error_handler, no_exit_handler
