@@ -80,6 +80,7 @@ class Context(AbstractContextManager):  # Extending AbstractContextManager to ma
     multiple_action_flags = ConfigOption()
     action_after_action_flags = ConfigOption()
     always_run_after_main = ConfigOption()
+    use_type_metavar = ConfigOption()
     # strict_option_punctuation = ConfigOption()
     # strict_action_punctuation = ConfigOption()
     # strict_sub_command_punctuation = ConfigOption()
@@ -97,6 +98,7 @@ class Context(AbstractContextManager):  # Extending AbstractContextManager to ma
         multiple_action_flags: Bool = None,
         action_after_action_flags: Bool = None,
         always_run_after_main: Bool = None,
+        use_type_metavar: Bool = None,
         # strict_option_punctuation: Bool = None,
         # strict_action_punctuation: Bool = None,
         # strict_sub_command_punctuation: Bool = None,
@@ -105,6 +107,7 @@ class Context(AbstractContextManager):  # Extending AbstractContextManager to ma
         self.remaining = list(self.argv)
         self.command = command
         self.parent = parent
+        self.failed = False
         if parent is not None:
             self._parsing = parent._parsing.copy()
             self.unknown = parent.unknown.copy()
@@ -122,6 +125,7 @@ class Context(AbstractContextManager):  # Extending AbstractContextManager to ma
         self.multiple_action_flags = multiple_action_flags
         self.action_after_action_flags = action_after_action_flags
         self.always_run_after_main = always_run_after_main
+        self.use_type_metavar = use_type_metavar
         # self.strict_option_punctuation = strict_option_punctuation
         # self.strict_action_punctuation = strict_action_punctuation
         # self.strict_sub_command_punctuation = strict_sub_command_punctuation
@@ -220,7 +224,8 @@ class Context(AbstractContextManager):  # Extending AbstractContextManager to ma
 
     @property
     def before_main_actions(self) -> Iterator['ActionFlag']:
-        for action_flag in self.parsed_action_flags[1]:
+        flags = self.parsed_always_available_action_flags if self.failed else self.parsed_action_flags[1]
+        for action_flag in flags:
             self.actions_taken += 1
             yield action_flag
 
@@ -230,6 +235,14 @@ class Context(AbstractContextManager):  # Extending AbstractContextManager to ma
             self.actions_taken += 1
             yield action_flag
 
+    @cached_property
+    def parsed_always_available_action_flags(self) -> Tuple['ActionFlag', ...]:
+        parsing = self._parsing
+        try:
+            return tuple(p for p in self.params.always_available_action_flags if p in parsing)
+        except AttributeError:  # self.command is None
+            return ()
+
 
 def get_current_context(silent: bool = False) -> Optional[Context]:
     """
