@@ -1,4 +1,6 @@
 """
+The class that handles parsing input.
+
 :author: Doug Skrypa
 """
 
@@ -6,7 +8,6 @@ import logging
 from collections import deque
 from typing import TYPE_CHECKING, Optional, Any, Deque
 
-from .actions import help_action
 from .context import ctx
 from .exceptions import (
     UsageError,
@@ -20,6 +21,7 @@ from .parameters import BaseOption, Parameter, BasePositional
 
 if TYPE_CHECKING:
     from .core import CommandType
+    from .command_parameters import CommandParameters
 
 __all__ = ['CommandParser']
 log = logging.getLogger(__name__)
@@ -39,36 +41,53 @@ class CommandParser:
         params = ctx.params
         sub_cmd_param = params.sub_command
         if sub_cmd_param is not None and not sub_cmd_param.choices:
+            ctx.failed = True
             raise CommandDefinitionError(f'{ctx.command}.{sub_cmd_param.name} = {sub_cmd_param} has no sub Commands')
 
         cls()._parse_args()
-        for group in params.groups:
-            group.validate()
+        cls._validate_groups(params)
 
         if sub_cmd_param is not None:
             try:
                 next_cmd = sub_cmd_param.result()  # type: CommandType
             except UsageError:
-                if help_action not in ctx:
+                ctx.failed = True
+                if not ctx.parsed_always_available_action_flags:
                     raise
             else:
                 missing = params.missing()
                 if missing and next_cmd.__class__.parent(next_cmd) is not ctx.command:
-                    if help_action in ctx:
+                    ctx.failed = True
+                    if ctx.parsed_always_available_action_flags:
                         return None
                     raise ParamsMissing(missing)
                 return next_cmd
 
         missing = params.missing()
         if missing and not ctx.allow_missing and (not params.action or params.action not in missing):
+            ctx.failed = True
             # Action is excluded because it provides a better error message
-            if help_action not in ctx:
+            if not ctx.parsed_always_available_action_flags:
                 raise ParamsMissing(missing)
         elif ctx.remaining and not ctx.ignore_unknown:
+            ctx.failed = True
             raise NoSuchOption('unrecognized arguments: {}'.format(' '.join(ctx.remaining)))
 
         return None
 
+    @classmethod
+    def _validate_groups(cls, params: 'CommandParameters'):
+        exc = None
+        for group in params.groups:
+            try:
+                group.validate()
+            except ParamsMissing as e:  # Let ParamConflict propagate before ParamsMissing
+                if exc is None:
+                    exc = e
+
+        if exc is not None:
+            raise exc
+
     def _parse_args(self):
         self.arg_deque = arg_deque = self.handle_pass_thru()
         self.deferred = ctx.remaining = []
